//////////////////////////////////////////////////////////////////////////////

#include "linkio.cpp"

//////////////////////////////////////////////////////////////////////////////

#define atoh(s) strtoul(s, 0, 16); // converts hexadecimal string to long

//////////////////////////////////////////////////////////////////////////////
// main - Main program                                                      //
//////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
  if (argc<2) {
    /////////// help /////////////
    printf("GameBoy link util v1.0 - Copyright (c) 1998 BlackThunder\n\n");
    printf("%s [filename] [-a<address>] [-x]\n\n", argv[0]);
    printf("  filename        Name of the file to upload.\n");
    printf("  -a<address>     Upload address in hex. Default=0xC000.\n");
    printf("  -x[<address>]   Execute: Optional address in hex. Default=0xC100.\n");
    printf("  -f              Upload trailing FF's.\n");
//    printf("  -s[<size>]      Size to upload.\n");
    return 1;
  }

  /////////// settings /////////////
  #define UPLOAD_DEFAULT 0xC000
  #define EXEC_DEFAULT 0xC100
  char *filename = 0;
  unsigned upload_addr = UPLOAD_DEFAULT;
  int execute = 0;
  unsigned exec_addr;
  long size = -1; // no trailing FF's

  /////////// scan arguments /////////////
  for (int a=1; a<argc; a++) {

    if (argv[a][0] == '-') {
      ////////// option //////////
      switch (argv[a][1]) {
	case 'a': { // upload address
	  upload_addr = atoh(argv[a]+2);
	  if (!upload_addr) upload_addr = UPLOAD_DEFAULT;
	  break;
	}
	case 'x': { // execution address
	  exec_addr = atoh(argv[a]+2);
	  if (!exec_addr) exec_addr = EXEC_DEFAULT;
	  execute = 1;
	  break;
	}
	case 'f': { // upload trailing FF's
	  size = 0; // use filelength
	  break;
	}
	default: { // unknown option
	  printf("Unknown option: '%s'\n", argv[a]);
	  return 1;
	}
      }
    } else {
      //////// filename /////////
      if (filename) {
	printf("Only one filename is allowed!\n");
	return 1;
      }
      filename = argv[a];
    }

  } // arguments

  if (filename) {
    printf("Uploading \"%s\" to $%04X...\n", filename, upload_addr);
    ReturnOnError(UploadFile(argv[1], upload_addr, size));
    printf("Uploaded %d bytes!\n", size);
  } else if (!execute) {
    printf("A filename is required!\n");
    return 1;
  }
  if (execute) {
    printf("Executing from $%04X...\n", exec_addr);
    ReturnOnError(Execute(exec_addr));
  }

  printf("\n");
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
