;----------------------------------------------------------------------------;
;DP = displacement
;----------------------------------------------------------------------------;

IF !DEF(DP)
DP = 0
ENDC

;----------------------------------------------------------------------------;

.CmdLoop:
  call  .Command+DP
  jr    .CmdLoop

.Command:
  ld    a, $1B
  call  .Transmit+DP
  cp    a, $E4
  ret   nz
  ld    a, $E4
  call  .Transmit+DP
  cp    a, $1B
  ret   nz
  call  .Transmit+DP
  cp    a, "r"
  jr    z, .Read        ;read block from memory
  cp    a, "w"
  jr    z, .Write       ;write block to memory
  cp    a, "x"
  jr    z, .Execute     ;execute program
  cp    a, "s"
  jp    z, SwitchToRAM  ;run from RAM
ret

;----------------------------------------------------------------------------;

.Block:                 ;read block address/size
  call  .Transmit+DP
  ld    h, a
  call  .Transmit+DP
  ld    l, a
  call  .Transmit+DP
  ld    b, a
  call  .Transmit+DP
  ld    c, a
ret

;----------------------------------------------------------------------------;

.Read:                  ;read from memory
  call  .Block+DP
  ld    e, 0            ;checksum
.readloop:
  vld   a, [hl+]
  ldh   [_SB], a        ;output
  add   a, e            ;apply checksum
  ld    e, a            ;save checksum
  call  .Transmit2+DP
  dec   bc
  ld    a, b
  or    a, c
  jr    nz, .readloop
  jp    .TransmitChecksum+DP

;----------------------------------------------------------------------------;

.Write:                 ;write to memory
  call  .Block+DP
  ld    e, 0            ;checksum
.writeloop:
  call  .Transmit2+DP
  vld   [hl+], a
  add   a, e            ;apply checksum
  ld    e, a            ;save checksum
  dec   bc
  ld    a, b
  or    a, c
  jr    nz, .writeloop
  jp    .TransmitChecksum+DP

;----------------------------------------------------------------------------;

.Execute:               ;execute program
  call  .Transmit+DP
  ld    h, a
  call  .Transmit+DP
  ld    l, a
  jp    [hl]

;----------------------------------------------------------------------------;

.TransmitChecksum:      ;output checksum
  ld    a, e
.Transmit:              ;output & input
  ldh   [_SB], a
.Transmit2:             ;user output & input
  ld    a, 128
  ldh   [_SC], a
.wait4data:
  ldh   a, [_SC]
  and   a, 128
  jr    nz, .wait4data  ;wait for data
  ldh   a, [_SB]        ;input
ret

;----------------------------------------------------------------------------;
