//////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>

//////////////////////////////////////////////////////////////////////////////
// Error codes and macro's                                                  //
//////////////////////////////////////////////////////////////////////////////

#define ERR_UNKNOWN 1
#define ERR_COMM 2
#define ERR_CRC 3
#define ERR_FILE 4
#define ERR_MEM 5
char *errors[6] = {"None", "Unknown", "Communication error", "CRC mismatch", "Can't open file", "Not enough memory"};
#define ShowError(n) printf("Error: %s.\n", errors[n]);
#define ReturnOnError(what) { int err = what; if (err) { ShowError(err); printf("\n"); return err; } }

//////////////////////////////////////////////////////////////////////////////
// Transfer - Transfer a byte from/to the GameBoy                           //
//////////////////////////////////////////////////////////////////////////////

char Transfer(char out)
{
  char in=0;
  for (int bit=7; bit>=0; bit--) {
    outp(0x378, out>>bit&1);
    outp(0x378, out>>bit&1 | 2);
    in += (inp(0x379)>>5&1)<<bit;
    outp(0x378, 0);
  }
  delay(1);
  return in;
}

//////////////////////////////////////////////////////////////////////////////
// Command - Send communication command                                     //
//////////////////////////////////////////////////////////////////////////////

int Command(char cmd)
{
  Transfer(0x1B);
  if (Transfer(0xE4) != 0x1B) return ERR_COMM;
  if (Transfer(0x1B) != 0xE4) return ERR_COMM;
  Transfer(cmd);
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
// Block - Specify block information                                        //
//////////////////////////////////////////////////////////////////////////////

int Block(unsigned a, unsigned size)
{
  Transfer(a>>8);
  Transfer(a);
  Transfer(size>>8);
  Transfer(size);
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
// Read - Download datablock                                                //
//////////////////////////////////////////////////////////////////////////////

int Read(char data[], unsigned a, unsigned size)
{
  if (Command('r')) return ERR_COMM;
  Block(a, size);
  char crc=0, *d=data;
  for (; size>0; size--) { crc += (*d++ = Transfer(0)); }
  if (crc != Transfer(0)) return ERR_CRC;
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
// Write - Upload datablock                                                 //
//////////////////////////////////////////////////////////////////////////////

int Write(char data[], unsigned a, unsigned size)
{
  if (Command('w')) return ERR_COMM;
  if (Block(a, size)) return ERR_COMM;
  char crc=0, *d=data;
  for (; size>0; size--) { crc += *d; Transfer(*d++); }
  if (crc != Transfer(0)) return ERR_CRC;
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
// ReadByte - Read one byte                                                 //
//////////////////////////////////////////////////////////////////////////////

int ReadByte(unsigned a, char &d)
{
  return Read(&d,a,1);
}

//////////////////////////////////////////////////////////////////////////////
// WriteByte - Write one byte                                               //
//////////////////////////////////////////////////////////////////////////////

int WriteByte(unsigned a, char d)
{
  return Write(&d,a,1);
}

//////////////////////////////////////////////////////////////////////////////
// Execute - Jump to specified address                                      //
//////////////////////////////////////////////////////////////////////////////

int Execute(unsigned a)
{
  if (Command('x')) return ERR_COMM;
  Transfer(a>>8);
  Transfer(a);
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
// UploadFile - Upload file to specified address                            //
// size: -1=skip 0xFF's, 0=filelength, >0=specified size                    //
//////////////////////////////////////////////////////////////////////////////

int UploadFile(char *fname, unsigned a, long &size)
{
  FILE *f = fopen(fname, "rb");
  if (!f) return ERR_FILE;
  if (size==-1) {				// skip 0xFF's
    while (!feof(f)) {
      char c; fread(&c,1,1,f);
      if (c != 0xFF) size = ftell(f);
    }
  } else if (size==0) {				// use filelength
    fseek(f, 0, SEEK_END); size = ftell(f);
  }
  char *data = (char*)malloc(size);
  if (!data) { fclose(f); return ERR_MEM; }
  fseek(f, 0, SEEK_SET);
  fread(data, 1, size, f);
  fclose(f);
  int err = Write(data, a, size);
  free(data);
  return err;
}

//////////////////////////////////////////////////////////////////////////////
// DownloadFile - Download memory block to file                             //
//////////////////////////////////////////////////////////////////////////////

int DownloadFile(char *fname, unsigned a, long size)
{
  char *data = (char*)malloc(size);
  if (!data) return ERR_MEM;
  int err = Read(data, a, size);
  if (err) return err;
  FILE *f = fopen(fname, "wb");
  if (!f) return ERR_FILE;
  fwrite(data, 1, size, f);
  fclose(f);
  free(data);
  return 0;
}

//////////////////////////////////////////////////////////////////////////////
